@@ -3,26 +3,39 @@
  * @ingroup MAT
  */
 /*
- * Copyright (C) 2005-2006   Christopher C. Hulbert
+ * Copyright (C) 2005-2011   Christopher C. Hulbert
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation; either
- *  version 2.1 of the License, or (at your option) any later version.
+ * All rights reserved.
  *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ *    1. Redistributions of source code must retain the above copyright notice,
+ *       this list of conditions and the following disclaimer.
+ *
+ *    2. Redistributions in binary form must reproduce the above copyright
+ *       notice, this list of conditions and the following disclaimer in the
+ *       documentation and/or other materials provided with the distribution.
+ *
+ * THIS SOFTWARE IS PROVIDED BY CHRISTOPHER C. HULBERT ``AS IS'' AND ANY
+ * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+ * DISCLAIMED. IN NO EVENT SHALL CHRISTOPHER C. HULBERT OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <stdlib.h>
-#include "matio.h"
 #include "matio_private.h"
 
+#if HAVE_ZLIB
+
+/** @cond mat_devman */
+
 /** @brief Inflate the data until @c nbytes of uncompressed data has been
  *         inflated
  *
@@ -32,7 +45,7 @@
  * @param nbytes Number of uncompressed bytes to skip
  * @return Number of bytes read from the file
  */
-int 
+int
 InflateSkip(mat_t *mat, z_stream *z, int nbytes)
 {
     mat_uint8_t comp_buf[512],uncomp_buf[512];
@@ -108,44 +121,44 @@ InflateSkip2(mat_t *mat, matvar_t *matvar, int nbytes)
     mat_uint8_t comp_buf[32],uncomp_buf[32];
     int     bytesread = 0, err, cnt = 0;
 
-    if ( !matvar->z->avail_in ) {
-        matvar->z->avail_in = 1;
-        matvar->z->next_in = comp_buf;
+    if ( !matvar->internal->z->avail_in ) {
+        matvar->internal->z->avail_in = 1;
+        matvar->internal->z->next_in = comp_buf;
         bytesread += fread(comp_buf,1,1,mat->fp);
     }
-    matvar->z->avail_out = 1;
-    matvar->z->next_out = uncomp_buf;
-    err = inflate(matvar->z,Z_NO_FLUSH);
+    matvar->internal->z->avail_out = 1;
+    matvar->internal->z->next_out = uncomp_buf;
+    err = inflate(matvar->internal->z,Z_NO_FLUSH);
     if ( err != Z_OK ) {
         Mat_Critical("InflateSkip2: %s - inflate returned %d",matvar->name,err);
         return bytesread;
     }
-    if ( !matvar->z->avail_out ) {
-        matvar->z->avail_out = 1;
-        matvar->z->next_out = uncomp_buf;
+    if ( !matvar->internal->z->avail_out ) {
+        matvar->internal->z->avail_out = 1;
+        matvar->internal->z->next_out = uncomp_buf;
     }
     while ( cnt < nbytes ) {
-        if ( !matvar->z->avail_in ) {
-            matvar->z->avail_in = 1;
-            matvar->z->next_in = comp_buf;
+        if ( !matvar->internal->z->avail_in ) {
+            matvar->internal->z->avail_in = 1;
+            matvar->internal->z->next_in = comp_buf;
             bytesread += fread(comp_buf,1,1,mat->fp);
             cnt++;
         }
-        err = inflate(matvar->z,Z_NO_FLUSH);
+        err = inflate(matvar->internal->z,Z_NO_FLUSH);
         if ( err != Z_OK ) {
             Mat_Critical("InflateSkip2: %s - inflate returned %d",matvar->name,err);
             return bytesread;
         }
-        if ( !matvar->z->avail_out ) {
-            matvar->z->avail_out = 1;
-            matvar->z->next_out = uncomp_buf;
+        if ( !matvar->internal->z->avail_out ) {
+            matvar->internal->z->avail_out = 1;
+            matvar->internal->z->next_out = uncomp_buf;
         }
     }
 
-    if ( matvar->z->avail_in ) {
-        fseek(mat->fp,-(int)matvar->z->avail_in,SEEK_CUR);
-        bytesread -= matvar->z->avail_in;
-        matvar->z->avail_in = 0;
+    if ( matvar->internal->z->avail_in ) {
+        fseek(mat->fp,-(int)matvar->internal->z->avail_in,SEEK_CUR);
+        bytesread -= matvar->internal->z->avail_in;
+        matvar->internal->z->avail_in = 0;
     }
 
     return bytesread;
@@ -162,7 +175,7 @@ InflateSkip2(mat_t *mat, matvar_t *matvar, int nbytes)
  * @return Number of bytes read from the file
  */
 int
-InflateSkipData(mat_t *mat,z_stream *z,int data_type,int len)
+InflateSkipData(mat_t *mat,z_stream *z,enum matio_types data_type,int len)
 {
     int data_size = 0;
 
@@ -206,6 +219,8 @@ InflateSkipData(mat_t *mat,z_stream *z,int data_type,int len)
         case MAT_T_INT8:
             data_size = sizeof(mat_int8_t);
             break;
+        default:
+            return 0;
     }
     InflateSkip(mat,z,len*data_size);
     return len;
@@ -217,7 +232,7 @@ InflateSkipData(mat_t *mat,z_stream *z,int data_type,int len)
  * @ingroup mat_internal
  * @param mat Pointer to the MAT file
  * @param matvar Pointer to the MAT variable
- * @param buf Pointer to store the 8-byte variable tag 
+ * @param buf Pointer to store the 8-byte variable tag
  * @return Number of bytes read from the file
  */
 int
@@ -229,33 +244,33 @@ InflateVarTag(mat_t *mat, matvar_t *matvar, void *buf)
     if (buf == NULL)
         return 0;
 
-    if ( !matvar->z->avail_in ) {
-        matvar->z->avail_in = 1;
-        matvar->z->next_in = comp_buf;
+    if ( !matvar->internal->z->avail_in ) {
+        matvar->internal->z->avail_in = 1;
+        matvar->internal->z->next_in = comp_buf;
         bytesread += fread(comp_buf,1,1,mat->fp);
     }
-    matvar->z->avail_out = 8;
-    matvar->z->next_out = buf;
-    err = inflate(matvar->z,Z_NO_FLUSH);
+    matvar->internal->z->avail_out = 8;
+    matvar->internal->z->next_out = buf;
+    err = inflate(matvar->internal->z,Z_NO_FLUSH);
     if ( err != Z_OK ) {
         Mat_Critical("InflateVarTag: inflate returned %d",err);
         return bytesread;
     }
-    while ( matvar->z->avail_out && !matvar->z->avail_in ) {
-        matvar->z->avail_in = 1;
-        matvar->z->next_in = comp_buf;
+    while ( matvar->internal->z->avail_out && !matvar->internal->z->avail_in ) {
+        matvar->internal->z->avail_in = 1;
+        matvar->internal->z->next_in = comp_buf;
         bytesread += fread(comp_buf,1,1,mat->fp);
-        err = inflate(matvar->z,Z_NO_FLUSH);
+        err = inflate(matvar->internal->z,Z_NO_FLUSH);
         if ( err != Z_OK ) {
             Mat_Critical("InflateVarTag: inflate returned %d",err);
             return bytesread;
         }
     }
 
-    if ( matvar->z->avail_in ) {
-        fseek(mat->fp,-(int)matvar->z->avail_in,SEEK_CUR);
-        bytesread -= matvar->z->avail_in;
-        matvar->z->avail_in = 0;
+    if ( matvar->internal->z->avail_in ) {
+        fseek(mat->fp,-(int)matvar->internal->z->avail_in,SEEK_CUR);
+        bytesread -= matvar->internal->z->avail_in;
+        matvar->internal->z->avail_in = 0;
     }
 
     return bytesread;
@@ -267,7 +282,7 @@ InflateVarTag(mat_t *mat, matvar_t *matvar, void *buf)
  * @ingroup mat_internal
  * @param mat Pointer to the MAT file
  * @param matvar Pointer to the MAT variable
- * @param buf Pointer to store the 16-byte array flags tag and data 
+ * @param buf Pointer to store the 16-byte array flags tag and data
  * @return Number of bytes read from the file
  */
 int
@@ -278,33 +293,33 @@ InflateArrayFlags(mat_t *mat, matvar_t *matvar, void *buf)
 
     if (buf == NULL) return 0;
 
-    if ( !matvar->z->avail_in ) {
-        matvar->z->avail_in = 1;
-        matvar->z->next_in = comp_buf;
+    if ( !matvar->internal->z->avail_in ) {
+        matvar->internal->z->avail_in = 1;
+        matvar->internal->z->next_in = comp_buf;
         bytesread += fread(comp_buf,1,1,mat->fp);
     }
-    matvar->z->avail_out = 16;
-    matvar->z->next_out = buf;
-    err = inflate(matvar->z,Z_NO_FLUSH);
+    matvar->internal->z->avail_out = 16;
+    matvar->internal->z->next_out = buf;
+    err = inflate(matvar->internal->z,Z_NO_FLUSH);
     if ( err != Z_OK ) {
         Mat_Critical("InflateArrayFlags: inflate returned %d",err);
         return bytesread;
     }
-    while ( matvar->z->avail_out && !matvar->z->avail_in ) {
-        matvar->z->avail_in = 1;
-        matvar->z->next_in = comp_buf;
+    while ( matvar->internal->z->avail_out && !matvar->internal->z->avail_in ) {
+        matvar->internal->z->avail_in = 1;
+        matvar->internal->z->next_in = comp_buf;
         bytesread += fread(comp_buf,1,1,mat->fp);
-        err = inflate(matvar->z,Z_NO_FLUSH);
+        err = inflate(matvar->internal->z,Z_NO_FLUSH);
         if ( err != Z_OK ) {
             Mat_Critical("InflateArrayFlags: inflate returned %d",err);
             return bytesread;
         }
     }
 
-    if ( matvar->z->avail_in ) {
-        fseek(mat->fp,-(int)matvar->z->avail_in,SEEK_CUR);
-        bytesread -= matvar->z->avail_in;
-        matvar->z->avail_in = 0;
+    if ( matvar->internal->z->avail_in ) {
+        fseek(mat->fp,-(int)matvar->internal->z->avail_in,SEEK_CUR);
+        bytesread -= matvar->internal->z->avail_in;
+        matvar->internal->z->avail_in = 0;
     }
 
     return bytesread;
@@ -318,7 +333,7 @@ InflateArrayFlags(mat_t *mat, matvar_t *matvar, void *buf)
  * @ingroup mat_internal
  * @param mat Pointer to the MAT file
  * @param matvar Pointer to the MAT variable
- * @param buf Pointer to store the dimensions flag and data 
+ * @param buf Pointer to store the dimensions flag and data
  * @return Number of bytes read from the file
  */
 int
@@ -331,23 +346,23 @@ InflateDimensions(mat_t *mat, matvar_t *matvar, void *buf)
     if ( buf == NULL )
         return 0;
 
-    if ( !matvar->z->avail_in ) {
-        matvar->z->avail_in = 1;
-        matvar->z->next_in = comp_buf;
+    if ( !matvar->internal->z->avail_in ) {
+        matvar->internal->z->avail_in = 1;
+        matvar->internal->z->next_in = comp_buf;
         bytesread += fread(comp_buf,1,1,mat->fp);
     }
-    matvar->z->avail_out = 8;
-    matvar->z->next_out = buf;
-    err = inflate(matvar->z,Z_NO_FLUSH);
+    matvar->internal->z->avail_out = 8;
+    matvar->internal->z->next_out = buf;
+    err = inflate(matvar->internal->z,Z_NO_FLUSH);
     if ( err != Z_OK ) {
         Mat_Critical("InflateDimensions: inflate returned %d",err);
         return bytesread;
     }
-    while ( matvar->z->avail_out && !matvar->z->avail_in ) {
-        matvar->z->avail_in = 1;
-        matvar->z->next_in = comp_buf;
+    while ( matvar->internal->z->avail_out && !matvar->internal->z->avail_in ) {
+        matvar->internal->z->avail_in = 1;
+        matvar->internal->z->next_in = comp_buf;
         bytesread += fread(comp_buf,1,1,mat->fp);
-        err = inflate(matvar->z,Z_NO_FLUSH);
+        err = inflate(matvar->internal->z,Z_NO_FLUSH);
         if ( err != Z_OK ) {
             Mat_Critical("InflateDimensions: inflate returned %d",err);
             return bytesread;
@@ -370,33 +385,33 @@ InflateDimensions(mat_t *mat, matvar_t *matvar, void *buf)
         i = 0;
     rank+=i;
 
-    if ( !matvar->z->avail_in ) {
-        matvar->z->avail_in = 1;
-        matvar->z->next_in = comp_buf;
+    if ( !matvar->internal->z->avail_in ) {
+        matvar->internal->z->avail_in = 1;
+        matvar->internal->z->next_in = comp_buf;
         bytesread += fread(comp_buf,1,1,mat->fp);
     }
-    matvar->z->avail_out = rank;
-    matvar->z->next_out = (void *)((mat_int32_t *)buf+2);
-    err = inflate(matvar->z,Z_NO_FLUSH);
+    matvar->internal->z->avail_out = rank;
+    matvar->internal->z->next_out = (void *)((mat_int32_t *)buf+2);
+    err = inflate(matvar->internal->z,Z_NO_FLUSH);
     if ( err != Z_OK ) {
         Mat_Critical("InflateDimensions: inflate returned %d",err);
         return bytesread;
     }
-    while ( matvar->z->avail_out && !matvar->z->avail_in ) {
-        matvar->z->avail_in = 1;
-        matvar->z->next_in = comp_buf;
+    while ( matvar->internal->z->avail_out && !matvar->internal->z->avail_in ) {
+        matvar->internal->z->avail_in = 1;
+        matvar->internal->z->next_in = comp_buf;
         bytesread += fread(comp_buf,1,1,mat->fp);
-        err = inflate(matvar->z,Z_NO_FLUSH);
+        err = inflate(matvar->internal->z,Z_NO_FLUSH);
         if ( err != Z_OK ) {
             Mat_Critical("InflateDimensions: inflate returned %d",err);
             return bytesread;
         }
     }
 
-    if ( matvar->z->avail_in ) {
-        fseek(mat->fp,-(int)matvar->z->avail_in,SEEK_CUR);
-        bytesread -= matvar->z->avail_in;
-        matvar->z->avail_in = 0;
+    if ( matvar->internal->z->avail_in ) {
+        fseek(mat->fp,-(int)matvar->internal->z->avail_in,SEEK_CUR);
+        bytesread -= matvar->internal->z->avail_in;
+        matvar->internal->z->avail_in = 0;
     }
 
     return bytesread;
@@ -407,7 +422,7 @@ InflateDimensions(mat_t *mat, matvar_t *matvar, void *buf)
  * @ingroup mat_internal
  * @param mat Pointer to the MAT file
  * @param matvar Pointer to the MAT variable
- * @param buf Pointer to store the variables name tag 
+ * @param buf Pointer to store the variables name tag
  * @return Number of bytes read from the file
  */
 int
@@ -419,33 +434,33 @@ InflateVarNameTag(mat_t *mat, matvar_t *matvar, void *buf)
     if ( buf == NULL )
         return 0;
 
-    if ( !matvar->z->avail_in ) {
-        matvar->z->avail_in = 1;
-        matvar->z->next_in = comp_buf;
+    if ( !matvar->internal->z->avail_in ) {
+        matvar->internal->z->avail_in = 1;
+        matvar->internal->z->next_in = comp_buf;
         bytesread += fread(comp_buf,1,1,mat->fp);
     }
-    matvar->z->avail_out = 8;
-    matvar->z->next_out = buf;
-    err = inflate(matvar->z,Z_NO_FLUSH);
+    matvar->internal->z->avail_out = 8;
+    matvar->internal->z->next_out = buf;
+    err = inflate(matvar->internal->z,Z_NO_FLUSH);
     if ( err != Z_OK ) {
         Mat_Critical("InflateVarNameTag: inflate returned %d",err);
         return bytesread;
     }
-    while ( matvar->z->avail_out && !matvar->z->avail_in ) {
-        matvar->z->avail_in = 1;
-        matvar->z->next_in = comp_buf;
+    while ( matvar->internal->z->avail_out && !matvar->internal->z->avail_in ) {
+        matvar->internal->z->avail_in = 1;
+        matvar->internal->z->next_in = comp_buf;
         bytesread += fread(comp_buf,1,1,mat->fp);
-        err = inflate(matvar->z,Z_NO_FLUSH);
+        err = inflate(matvar->internal->z,Z_NO_FLUSH);
         if ( err != Z_OK ) {
             Mat_Critical("InflateVarNameTag: inflate returned %d",err);
             return bytesread;
         }
     }
 
-    if ( matvar->z->avail_in ) {
-        fseek(mat->fp,-(int)matvar->z->avail_in,SEEK_CUR);
-        bytesread -= matvar->z->avail_in;
-        matvar->z->avail_in = 0;
+    if ( matvar->internal->z->avail_in ) {
+        fseek(mat->fp,-(int)matvar->internal->z->avail_in,SEEK_CUR);
+        bytesread -= matvar->internal->z->avail_in;
+        matvar->internal->z->avail_in = 0;
     }
 
     return bytesread;
@@ -456,7 +471,7 @@ InflateVarNameTag(mat_t *mat, matvar_t *matvar, void *buf)
  * @ingroup mat_internal
  * @param mat Pointer to the MAT file
  * @param matvar Pointer to the MAT variable
- * @param buf Pointer to store the variables name 
+ * @param buf Pointer to store the variables name
  * @param N Number of characters in the name
  * @return Number of bytes read from the file
  */
@@ -469,33 +484,33 @@ InflateVarName(mat_t *mat, matvar_t *matvar, void *buf, int N)
     if ( buf == NULL )
         return 0;
 
-    if ( !matvar->z->avail_in ) {
-        matvar->z->avail_in = 1;
-        matvar->z->next_in = comp_buf;
+    if ( !matvar->internal->z->avail_in ) {
+        matvar->internal->z->avail_in = 1;
+        matvar->internal->z->next_in = comp_buf;
         bytesread += fread(comp_buf,1,1,mat->fp);
     }
-    matvar->z->avail_out = N;
-    matvar->z->next_out = buf;
-    err = inflate(matvar->z,Z_NO_FLUSH);
+    matvar->internal->z->avail_out = N;
+    matvar->internal->z->next_out = buf;
+    err = inflate(matvar->internal->z,Z_NO_FLUSH);
     if ( err != Z_OK ) {
         Mat_Critical("InflateVarName: inflate returned %d",err);
         return bytesread;
     }
-    while ( matvar->z->avail_out && !matvar->z->avail_in ) {
-        matvar->z->avail_in = 1;
-        matvar->z->next_in = comp_buf;
+    while ( matvar->internal->z->avail_out && !matvar->internal->z->avail_in ) {
+        matvar->internal->z->avail_in = 1;
+        matvar->internal->z->next_in = comp_buf;
         bytesread += fread(comp_buf,1,1,mat->fp);
-        err = inflate(matvar->z,Z_NO_FLUSH);
+        err = inflate(matvar->internal->z,Z_NO_FLUSH);
         if ( err != Z_OK ) {
             Mat_Critical("InflateVarName: inflate returned %d",err);
             return bytesread;
         }
     }
 
-    if ( matvar->z->avail_in ) {
-        fseek(mat->fp,-(int)matvar->z->avail_in,SEEK_CUR);
-        bytesread -= matvar->z->avail_in;
-        matvar->z->avail_in = 0;
+    if ( matvar->internal->z->avail_in ) {
+        fseek(mat->fp,-(int)matvar->internal->z->avail_in,SEEK_CUR);
+        bytesread -= matvar->internal->z->avail_in;
+        matvar->internal->z->avail_in = 0;
     }
 
     return bytesread;
@@ -507,7 +522,7 @@ InflateVarName(mat_t *mat, matvar_t *matvar, void *buf, int N)
  * @ingroup mat_internal
  * @param mat Pointer to the MAT file
  * @param matvar Pointer to the MAT variable
- * @param buf Pointer to store the data tag 
+ * @param buf Pointer to store the data tag
  * @return Number of bytes read from the file
  */
 int
@@ -519,25 +534,25 @@ InflateDataTag(mat_t *mat, matvar_t *matvar, void *buf)
     if ( buf == NULL )
         return 0;
 
-   if ( !matvar->z->avail_in ) {
-        matvar->z->avail_in = 1;
-        matvar->z->next_in = comp_buf;
+   if ( !matvar->internal->z->avail_in ) {
+        matvar->internal->z->avail_in = 1;
+        matvar->internal->z->next_in = comp_buf;
         bytesread += fread(comp_buf,1,1,mat->fp);
     }
-    matvar->z->avail_out = 8;
-    matvar->z->next_out = buf;
-    err = inflate(matvar->z,Z_NO_FLUSH);
+    matvar->internal->z->avail_out = 8;
+    matvar->internal->z->next_out = buf;
+    err = inflate(matvar->internal->z,Z_NO_FLUSH);
     if ( err == Z_STREAM_END ) {
         return bytesread;
     } else if ( err != Z_OK ) {
         Mat_Critical("InflateDataTag: %s - inflate returned %d",matvar->name,err);
         return bytesread;
     }
-    while ( matvar->z->avail_out && !matvar->z->avail_in ) {
-        matvar->z->avail_in = 1;
-        matvar->z->next_in = comp_buf;
+    while ( matvar->internal->z->avail_out && !matvar->internal->z->avail_in ) {
+        matvar->internal->z->avail_in = 1;
+        matvar->internal->z->next_in = comp_buf;
         bytesread += fread(comp_buf,1,1,mat->fp);
-        err = inflate(matvar->z,Z_NO_FLUSH);
+        err = inflate(matvar->internal->z,Z_NO_FLUSH);
         if ( err == Z_STREAM_END ) {
             break;
         } else if ( err != Z_OK ) {
@@ -546,10 +561,10 @@ InflateDataTag(mat_t *mat, matvar_t *matvar, void *buf)
         }
     }
 
-    if ( matvar->z->avail_in ) {
-        fseek(mat->fp,-(int)matvar->z->avail_in,SEEK_CUR);
-        bytesread -= matvar->z->avail_in;
-        matvar->z->avail_in = 0;
+    if ( matvar->internal->z->avail_in ) {
+        fseek(mat->fp,-(int)matvar->internal->z->avail_in,SEEK_CUR);
+        bytesread -= matvar->internal->z->avail_in;
+        matvar->internal->z->avail_in = 0;
     }
 
     return bytesread;
@@ -561,7 +576,7 @@ InflateDataTag(mat_t *mat, matvar_t *matvar, void *buf)
  * @ingroup mat_internal
  * @param mat Pointer to the MAT file
  * @param matvar Pointer to the MAT variable
- * @param buf Pointer to store the data type 
+ * @param buf Pointer to store the data type
  * @return Number of bytes read from the file
  */
 int
@@ -611,7 +626,7 @@ InflateDataType(mat_t *mat, z_stream *z, void *buf)
  * @ingroup mat_internal
  * @param mat Pointer to the MAT file
  * @param z zlib compression stream
- * @param buf Pointer to store the data type 
+ * @param buf Pointer to store the data type
  * @param nBytes Number of bytes to inflate
  * @return Number of bytes read from the file
  */
@@ -658,7 +673,7 @@ InflateData(mat_t *mat, z_stream *z, void *buf, int nBytes)
             bytesread  += z->avail_in;
             z->next_in  = comp_buf;
         } else {
-            z->avail_in = fread(comp_buf,1,nBytes,mat->fp);
+            z->avail_in = fread(comp_buf,1,nBytes-bytesread,mat->fp);
             bytesread  += z->avail_in;
             z->next_in  = comp_buf;
         }
@@ -687,7 +702,7 @@ InflateData(mat_t *mat, z_stream *z, void *buf, int nBytes)
  * @ingroup mat_internal
  * @param mat Pointer to the MAT file
  * @param matvar Pointer to the MAT variable
- * @param buf Pointer to store the fieldname length 
+ * @param buf Pointer to store the fieldname length
  * @return Number of bytes read from the file
  */
 int
@@ -699,33 +714,33 @@ InflateFieldNameLength(mat_t *mat, matvar_t *matvar, void *buf)
     if ( buf == NULL )
         return 0;
 
-    if ( !matvar->z->avail_in ) {
-        matvar->z->avail_in = 1;
-        matvar->z->next_in = comp_buf;
+    if ( !matvar->internal->z->avail_in ) {
+        matvar->internal->z->avail_in = 1;
+        matvar->internal->z->next_in = comp_buf;
         bytesread += fread(comp_buf,1,1,mat->fp);
     }
-    matvar->z->avail_out = 8;
-    matvar->z->next_out = buf;
-    err = inflate(matvar->z,Z_NO_FLUSH);
+    matvar->internal->z->avail_out = 8;
+    matvar->internal->z->next_out = buf;
+    err = inflate(matvar->internal->z,Z_NO_FLUSH);
     if ( err != Z_OK ) {
         Mat_Critical("InflateFieldNameLength: inflate returned %d",err);
         return bytesread;
     }
-    while ( matvar->z->avail_out && !matvar->z->avail_in ) {
-        matvar->z->avail_in = 1;
-        matvar->z->next_in = comp_buf;
+    while ( matvar->internal->z->avail_out && !matvar->internal->z->avail_in ) {
+        matvar->internal->z->avail_in = 1;
+        matvar->internal->z->next_in = comp_buf;
         bytesread += fread(comp_buf,1,1,mat->fp);
-        err = inflate(matvar->z,Z_NO_FLUSH);
+        err = inflate(matvar->internal->z,Z_NO_FLUSH);
         if ( err != Z_OK ) {
             Mat_Critical("InflateFieldNameLength: inflate returned %d",err);
             return bytesread;
         }
     }
 
-    if ( matvar->z->avail_in ) {
-        fseek(mat->fp,-(int)matvar->z->avail_in,SEEK_CUR);
-        bytesread -= matvar->z->avail_in;
-        matvar->z->avail_in = 0;
+    if ( matvar->internal->z->avail_in ) {
+        fseek(mat->fp,-(int)matvar->internal->z->avail_in,SEEK_CUR);
+        bytesread -= matvar->internal->z->avail_in;
+        matvar->internal->z->avail_in = 0;
     }
 
     return bytesread;
@@ -737,7 +752,7 @@ InflateFieldNameLength(mat_t *mat, matvar_t *matvar, void *buf)
  * @ingroup mat_internal
  * @param mat Pointer to the MAT file
  * @param matvar Pointer to the MAT variable
- * @param buf Pointer to store the fieldname tag 
+ * @param buf Pointer to store the fieldname tag
  * @return Number of bytes read from the file
  */
 int
@@ -749,33 +764,33 @@ InflateFieldNamesTag(mat_t *mat, matvar_t *matvar, void *buf)
     if ( buf == NULL )
         return 0;
 
-    if ( !matvar->z->avail_in ) {
-        matvar->z->avail_in = 1;
-        matvar->z->next_in = comp_buf;
+    if ( !matvar->internal->z->avail_in ) {
+        matvar->internal->z->avail_in = 1;
+        matvar->internal->z->next_in = comp_buf;
         bytesread += fread(comp_buf,1,1,mat->fp);
     }
-    matvar->z->avail_out = 8;
-    matvar->z->next_out = buf;
-    err = inflate(matvar->z,Z_NO_FLUSH);
+    matvar->internal->z->avail_out = 8;
+    matvar->internal->z->next_out = buf;
+    err = inflate(matvar->internal->z,Z_NO_FLUSH);
     if ( err != Z_OK ) {
         Mat_Critical("InflateFieldNamesTag: inflate returned %d",err);
         return bytesread;
     }
-    while ( matvar->z->avail_out && !matvar->z->avail_in ) {
-        matvar->z->avail_in = 1;
-        matvar->z->next_in = comp_buf;
+    while ( matvar->internal->z->avail_out && !matvar->internal->z->avail_in ) {
+        matvar->internal->z->avail_in = 1;
+        matvar->internal->z->next_in = comp_buf;
         bytesread += fread(comp_buf,1,1,mat->fp);
-        err = inflate(matvar->z,Z_NO_FLUSH);
+        err = inflate(matvar->internal->z,Z_NO_FLUSH);
         if ( err != Z_OK ) {
             Mat_Critical("InflateFieldNamesTag: inflate returned %d",err);
             return bytesread;
         }
     }
 
-    if ( matvar->z->avail_in ) {
-        fseek(mat->fp,-(int)matvar->z->avail_in,SEEK_CUR);
-        bytesread -= matvar->z->avail_in;
-        matvar->z->avail_in = 0;
+    if ( matvar->internal->z->avail_in ) {
+        fseek(mat->fp,-(int)matvar->internal->z->avail_in,SEEK_CUR);
+        bytesread -= matvar->internal->z->avail_in;
+        matvar->internal->z->avail_in = 0;
     }
 
     return bytesread;
@@ -791,7 +806,7 @@ InflateFieldNamesTag(mat_t *mat, matvar_t *matvar, void *buf)
  * @ingroup mat_internal
  * @param mat Pointer to the MAT file
  * @param matvar Pointer to the MAT variable
- * @param buf Pointer to store the fieldnames 
+ * @param buf Pointer to store the fieldnames
  * @param nfields Number of fields
  * @param fieldname_length Maximum length in bytes of each field
  * @param padding Number of padding bytes
@@ -807,34 +822,38 @@ InflateFieldNames(mat_t *mat,matvar_t *matvar,void *buf,int nfields,
     if ( buf == NULL )
         return 0;
 
-    if ( !matvar->z->avail_in ) {
-        matvar->z->avail_in = 1;
-        matvar->z->next_in = comp_buf;
+    if ( !matvar->internal->z->avail_in ) {
+        matvar->internal->z->avail_in = 1;
+        matvar->internal->z->next_in = comp_buf;
         bytesread += fread(comp_buf,1,1,mat->fp);
     }
-    matvar->z->avail_out = nfields*fieldname_length+padding;
-    matvar->z->next_out = buf;
-    err = inflate(matvar->z,Z_NO_FLUSH);
+    matvar->internal->z->avail_out = nfields*fieldname_length+padding;
+    matvar->internal->z->next_out = buf;
+    err = inflate(matvar->internal->z,Z_NO_FLUSH);
     if ( err != Z_OK ) {
         Mat_Critical("InflateFieldNames: inflate returned %d",err);
         return bytesread;
     }
-    while ( matvar->z->avail_out && !matvar->z->avail_in ) {
-        matvar->z->avail_in = 1;
-        matvar->z->next_in = comp_buf;
+    while ( matvar->internal->z->avail_out && !matvar->internal->z->avail_in ) {
+        matvar->internal->z->avail_in = 1;
+        matvar->internal->z->next_in = comp_buf;
         bytesread += fread(comp_buf,1,1,mat->fp);
-        err = inflate(matvar->z,Z_NO_FLUSH);
+        err = inflate(matvar->internal->z,Z_NO_FLUSH);
         if ( err != Z_OK ) {
             Mat_Critical("InflateFieldNames: inflate returned %d",err);
             return bytesread;
         }
     }
 
-    if ( matvar->z->avail_in ) {
-        fseek(mat->fp,-(int)matvar->z->avail_in,SEEK_CUR);
-        bytesread -= matvar->z->avail_in;
-        matvar->z->avail_in = 0;
+    if ( matvar->internal->z->avail_in ) {
+        fseek(mat->fp,-(int)matvar->internal->z->avail_in,SEEK_CUR);
+        bytesread -= matvar->internal->z->avail_in;
+        matvar->internal->z->avail_in = 0;
     }
 
     return bytesread;
 }
+
+/** @endcond */
+
+#endif
