@@ -28,7 +28,7 @@
 #ifndef MAT73_H
 #define MAT73_H
 
-#include "hdf5.h"
+#include <hdf5.h>
 
 #ifdef __cplusplus
 #   define EXTERN extern "C"
@@ -42,6 +42,8 @@ EXTERN void      Mat_VarPrint73(matvar_t *matvar,int printdata);
 EXTERN void      Mat_VarRead73(mat_t *mat,matvar_t *matvar);
 EXTERN int       Mat_VarReadData73(mat_t *mat,matvar_t *matvar,void *data,
                      int *start,int *stride,int *edge);
+EXTERN int       Mat_VarReadDataLinear73(mat_t *mat,matvar_t *matvar,void *data,
+                     int start,int stride,int edge);
 EXTERN matvar_t *Mat_VarReadNextInfo73(mat_t *mat);
 EXTERN int       Mat_VarWrite73(mat_t *mat,matvar_t *matvar,int compress);
 
