@@ -35,6 +35,11 @@
 #if !defined(HAVE_STRCASECMP)
 #   define strcasecmp(a,b) strcmp(a,b)
 #endif
+#ifdef _MSC_VER
+#   define SIZE_T_FMTSTR "Iu"
+#else
+#   define SIZE_T_FMTSTR "zu"
+#endif
 
 static const char *optstring = "df:hvHV";
 static struct option options[] = {
@@ -227,8 +232,6 @@ slab_select_valid(int rank,int *start,int *stride,int *edge,matvar_t *matvar)
     } else if ( rank == 1 ) {
         for ( i = 0; i < matvar->rank; i++ )
             nmemb *= matvar->dims[i];
-        if ( edge[0] < 0 )
-            edge[0] = (nmemb-start[i]) / stride[0];
         if ( *stride < 1 ) {
             /* Check stride is at least 1 */
             fprintf(stderr,"stride must be positive");
@@ -377,8 +380,9 @@ read_selected_data(mat_t *mat,matvar_t *matvar,char *index_str)
                     } else if (matvar->rank == 2 && matvar->dims[1] == 1) {
                        matvar->dims[0] = *edge;
                     } else {
-                       matvar->rank = 1;
-                       matvar->dims[0] = *edge;
+                        matvar->rank = 2;
+                        matvar->dims[0] = *edge;
+                        matvar->dims[1] = 1;
                     }
                 } else {
                     Mat_VarReadData(mat,matvar,matvar->data,start,stride,edge);
@@ -537,11 +541,11 @@ print_whos(matvar_t *matvar)
     printf("%-20s", matvar->name);
     if ( matvar->rank > 0 ) {
         int cnt = 0;
-        printf("%8zu", matvar->dims[0]);
+        printf("%8" SIZE_T_FMTSTR, matvar->dims[0]);
         nbytes = matvar->dims[0];
         for ( i = 1; i < matvar->rank; i++ ) {
             if ( ceil(log10(matvar->dims[i]))+1 < 32 )
-                cnt += sprintf(size+cnt,"x%zu", matvar->dims[i]);
+                cnt += sprintf(size+cnt,"x%" SIZE_T_FMTSTR, matvar->dims[i]);
             nbytes *= matvar->dims[i];
         }
         printf("%-10s",size);
